# Overview

The frontend is designed as an single page application (SPA)

It has only one main html file and load some big js files (which are the real app) to modify the dom

The main js framework used is `react` and for styles/components `semantic ui react` is used

For state management/communication between components `redux` is used

Because YAPeX is a SPA only the data needs to be transferred between the frontend and backend
For this purpose we have dedicated classes which don't depend on `react` nor `semantic ui`, see [communication](../core/communication.md)

## Immutability

Hope this is the right article (didn't remember properly) [https://hackernoon.com/functional-programming-paradigms-in-modern-javascript-immutability-4e9751ca005c](https://hackernoon.com/functional-programming-paradigms-in-modern-javascript-immutability-4e9751ca005c) which describes good why we use immutability here...

basically we don't want to deal with the reference stuff anymore

With a single source of truth (single storage, see [redux](../tutorials/redux.md)) we have a lot of references to that store and many components use the same data in the storage. If a component would mutate the storage it could be surprising why data is changed inside other components (because of the reference).

Also the functional style is more descriptive than the imperative one. 
Performance wise this might be worse but the code should be more easily to read & write.

## Workflow

See [workflow](workflow.md) for the workflow of the app...
which file is the entry point, and what is called & done from there one (e.g. login)

## Structure

The frontend has some special files and folders which are explained here

- `customChangedLib/` some frameworks/libraries don't include typescript typings we need to specify them self
	for every lib a separate directory
	if we need to rewrite some files inside a lib place the new version inside the appropriate folder for the lib inside `customChangedLib/`
- `dist/` the output directory of the app
- `externalLibsForBundling/` some libs are not available via npm then they can be stored here
	actually some changes to libs are also stored here [TODO maybe merge or truly separate externalLibsForBundling/ and customChangedLib/ ??]
- `htmlTemplates/` contains the templates for the main html file for the app
	there are 2 because one is for developing and one for deployment
	[ejs](http://www.embeddedjs.com) is used as template language, can be changed inside `webpack.config.dist.js` in the `HtmlWebpackPlugin`
- `i18n/` is for internationalization stuff, see [i18n](../core/i18n.md)
- `icons/` contains the image and project files for all used images (e.g. the logo)
	.afdesign files can be opened with [affinity designer](https://affinity.serif.com/de/designer/) as an alternative for adobe indesign
- `node_modules/` contains all node modules (libs)
- `scripts/` contains scripts to perform some tasks
- `src/` contains the main app
- `static_pages/` contains some static pages that are separate but part of the frontend, see [static pages](../core/staticPages.md)
- `typings/` contains typings, managed by typings (typescript typing manager) but obsolete...
- `.gitignore` gitignore file
- `count.ignore` can be ignored... used with a private program to count the lines in the project (this file contains the paths to ignore)
- `debug.ts` controls if the app runs in debug mode or not
- `FRAGEN.md` contains some questions that are not answered yet for features, fixes, ... that should be some time asked to the admins
- `package.json` the package file
- `readme.md` a small setup guide
- `TODO.md` some ideas for the future
- `TODOStyle.md` some style ideas
- `tsconfig.json` the config file for the typescript compiler
- `tslint.json` the ts lint config file (to configure some linting rules)
- `typeFixes.md` some fixes for some type definitions that were not fixed in the official typings
- `webpack.config.dist.js` the webpack config file for deployment
- `webpack.config.js` the webpack config file for developing
	this is never referenced only `webpack.config.dist.js` so the right one should be chosen depending on the context

Because the sources of the app are mainly located in `frontend/src/` you can read more about the *inner* structure here [inner structure](innerStructure.md)