# Workflow


## Login

Everything starts with the user opening the Url to YAPEX.

The browser gets the *only* (except the static pages) html site for the app which contains a reference to our bundled js files.

React then starts the app with the file `frontend/src/index.tsx`

First the setup is done, see [setup](../core/setup.md)

After that the root component is created and added to the DOM

The `rootStorage` is also created now, because there is no action every reducer returns its initial state

the `frontend/src/components/login/loginOverlay.tsx` is created which makes a request to the backend and checks if the user is already logged in.

A user is logged in if he has the auth cookie and a csrf token set.
For more information about how the login in handled (what steps a third-party site has to do for login) see [Tutorial SigningIn](../tutorials/signingIn.md)

The checking is done by trying to get the user data.

If this fails the login component (at `frontend/src/components/login`) is displayed. When the user logs in the auth cookie is set by the backend (the backend requests the browser to create the cookie).

If this succeeds the login is hidden and the app component is displayed


The app component contains all other site components.

Each site component is wrapped in a react router component which will render the side of the path matches.


## External login

It's also possible that users come from OSTEPU (or other services in the future).

When a user is logged into OSTEPU and clicks on the YAPEX link...
The link looks like `https://yapex.informatik.uni-halle.de/api/auth/login/external?tid=...`
The `tid` is set from OSTEPU

The Auth controller in the backend has a function to handle the request named `LoginExternal`

The `tid` from OSTEPU is taken to make requests against OSTEPU.

The backend use the `tid` and checks if the user is logged into OSTEPU.

If that's the case we use the `tid` to get the user data from OSTEPU (e.g. firstName, lastName, ...)

We the check if the external user already exists in YAPEX

- if yes, check if there is a connected YAPEX user for the external user

	- if no return error (this can be the case if a user is deleted in YAPEX but kept in OSTEPU)
	- if yes, check if the user's password is empty or null or whitespace characters

		- if yes redirect to set password page
		- if no set cookie and redirect to YAPEX (we have an external user, YAPEX user and the user has a password)
		
- if no (no external user found)
	- check if there already exists a YAPEX user with the given name (login token)

		- if yes
				- copy the values from OSTEPU data into our YAPEX user data
				- create an external user
		- if no (no external user, no YAPEX user)
				- create an external user
				- create a new user because OSTEPU has this user (the password is set to empty)
					- set the default role, add the user to the default group
					- redirect to set password page *(a brand-new user will end up here!)*


Because brand-new users will be created in YAPEX (and an external user) the user exists before the password is set in the set password site.

To disallow that an other user logs in during the user sets it's new password, we reject empty passwords for login. Actually the empty string is not even hashed to empty... so we can distinguish this


After the user has set the password in the set password website the new password (and the tid) is send back to the backend

The backend then checks again if the tid is valid in OSTEPU and gets the user data from OSTEPU again (name, ...) to get the YAPEX user (only from tid and pw we don't know which user this is)

After some checks are made (e.g. pw not empty), external user exists and YAPEX user exists
we set the password for the user.

If everything went well the frontend receives a 200 and redirects the user to the YAPEX login site. We don't immediately login the user after setting the password so that every used need to check if the login with the new password works


## Refresh user data (through OSTEPU)

TO do so just delete all external users

With the login logic above the YAPEX user exists but no external user thus a new external user will be created when the user is redirected from OSTEPU and the data from OSTEPU is copied into the YAPEX user.
