# Better List and Table rendering

As you know the components get re-rended if the props change

This also applies for list things like lists and tables

Consider this code (direct approach)

```ts
const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    list: props.list
  }
}

//...

class SomeListComp extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>
        <Table sortable selectable padded striped singleLine className="material-table">
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell>
                <span>Item</span>
              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              this.props.list.map((item, index) => {
                return (
                  <Table.Row key={index}>
                      <Table.Cell>
                        <span>{item}</span>
                      </Table.Cell>
                  </Table.Row>
                )
              })
          </Table.Body>
        </Table>
      </div>
    )
  }
}
```


The *problem* is that if the props change react checks all child components if they need to be re-rendered too
So in this example there are 0 child components so the table will be fully re-rendered

If we use a child component and use it as a row like (indirect approach)

```ts
class SomeListComp extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>
        <Table sortable selectable padded striped singleLine className="material-table">
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell>
                <span>Item</span>
              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              this.props.list.map((item, index) => {
                return (
                  <MyRowComp key={index} data={item} />>
                )
              })
          </Table.Body>
        </Table>
      </div>
    )
  }
}
```

React checks the child components `MyRowComp` if they should be re-rendered.

This works especially good when the *row* is more complicated (more columns, maybe nested components, ...)

[TODO if the amount of rows don't change this is really better (e.g. active users list) but if we e.g. delete a row? will this **really** work too?? maybe check]

## Conclusion

So for small lists (and if they will stay small in the future for all users) the *direct* approach is ok --> less files

For large lists better use the 2. *indirect* approach


See also this [link](https://stackoverflow.com/questions/38033442/big-list-performance-with-react)