# Deployment setup

The file describes how to setup the frontend used to deploy the app.

Actually YAPEX is build on top of platform independent technologies but some scripts are only available in unix (ubuntu/macos should work too)
Because of this the steps taken in the scripts are described here to reproduce them manually

Most of the time it is wanted that we setup backend, frontend & the other projects and for the structure it is helpful if we place them all inside one folder e.g. called `yapex`. We will refer to this common folder as `yapex` from now on

## 0 Requirements

You need to make sure you have the following programs installed
these should be accessible via the terminal/console without any path and by there common names (e.g. nodejs --> `node`)

- nodejs/npm
- git

## 1 Get the code

```bash
cd yapex
git clone https://gitlab.informatik.uni-halle.de/Syndrom/frontend-react.git
cd frontend-react
```

All further steps assume you are in `yapex/frontend-react/`

## 2 Make sure we are in deploy mode

Currently the file `frontend/deploy.ts` determines if we are in deploy mode and some port stuff.

You need to check the line

```ts
export const isProduction = false
```

that `isProduction` is set to `true`.

## 3 Get the libs

Because we use npm to manage libs and because there are sooooo many we don't ship them with the project (this saves us ~200MB of storage in gitlab)

So we download all libs with

```bash
npm install --production
```

this can take a while...

## 4 Build

To build the frontend just run the script located at `frontend/scripts/build.sh`

On windows (or if you don't want to use the script) do the following

compile all static pages, then build the frontend

```bash

cd static_pages\firstExternalLoginPage
..\..\node_modules\.bin/tsc firstLoginExternalLogin.ts

cd ..\passwordMigrationHelper
..\..\node_modules\.bin/tsc passwordMigrationHelper.ts

cd ..\..
npm run build
```

## 5 Grab the output

If everything went well (some warnings are ok) the output is located at `frontend/dist/`

If some backend docs mention the output of the frontend or just the frontend app e.g. for deployment everything inside `frontend/dist/` is meant


## 6 CORS

Because the frontend is served from webpack dev server at port 8080 and the backend runs on a different port (5000) the browser normally disallows the communication.
Because the frontend js tries to access a different origin.

See [https://spring.io/understanding/CORS](https://spring.io/understanding/CORS)

If the ports didn't change this is no problem. If it still don't work and you get some cors warning in the browser console do the following

In the backend check the `backend/src/ClientServer/appsettings.json` file and set the `DeployOriginUrls` option to the url used by server, e.g. `["https://yapex.informatik.uni-halle.de"]`, multiple values are allowed (json array)
